# Umbrella Library

This library provides reusable components for use inside the SRS "Umbrella Frontend." This library was generated with [Angular CLI](https://github.com/angular/angular-cli) version 12.2.0.


----

[TOC]


# Installation

To install and run this application locally, you need to have [NodeJS](https://nodejs.org/en/) installed at a version of `10.0` or greater.

To complete installation and run the app locally, please [follow these instructions](https://greenlots.atlassian.net/wiki/spaces/DEV/pages/449871881/Angular+App+Development+Local+Configuration+Setup).

# Publishing the Library
We use the Angular CLI and the npm package manager to build and publish the library as an npm package.

After building your library from the `projects` directory with `ng build umbrella-library`, go back out to the dist folder `cd ../dist/umbrella-library`, add a new version if needed, and run `npm publish`

*Note: You may need to use the public access flag when publishing: `npm publish --access=public`*

```
ng build umbrella-library
cd ../dist/umbrella-library
npm version [bump to new version]
npm publish
```

From [Angular documentation](https://angular.io/guide/creating-libraries#publishing-your-library): You should always build libraries for distribution using the production configuration. This ensures that generated output uses the appropriate optimizations and the correct package format for npm.

# Git Branching Model

We use git to manage all of our development flow.
*Please create branches and pull requests properly.* Read our [Git Branching Model](https://greenlots.atlassian.net/wiki/spaces/DEV/pages/1787396374/Driver+Portal+-+Git+Flow+Build+Processes) to accomplish this. (That document is specific to the Driver Portal, but the model is the same for this application).


# Code Scaffolding

Run `ng generate component component-name --project umbrella-library` to generate a new component. You can also use `ng generate directive|pipe|service|class|guard|interface|enum|module --project umbrella-library`.
-> Note: Don't forget to add `--project umbrella-library` or else it will be added to the default project in your `angular.json` file.


# Code Formatting

This application requires well-formatted code. It is recommended you set up automated code formatting. The easiest way to do this is to simply configure your IDE to format files on every save as follows:

### Automated Formatting

Configure your IDE to run the `format:all` command upon save. With VSCode, this can be done by installing a plugin such as [RunOnSave](https://marketplace.visualstudio.com/items?itemName=emeraldwalk.RunOnSave). If you do install RunOnSave, simply add this configuration to your VSCode's `settings.json`:

```
"emeraldwalk.runonsave": {
  "commands": [
    {
      "match": ".*",
      "cmd": "npm run format:all"
    }
  ]
}
```

### Manual Formatting

If you don't choose to set up auto-formatting, you can also manually format your code with :

**- Prettier (TS, CSS)**. We use [Prettier](https://prettier.io/) to format all files except for HTML (see below). Manually call `npm run format:prettier` to format all TS, CSS, and config files inside the `src` directory. This command is an alias for `npx prettier --write src`, which you can also use if you prefer not to use the alias, and you can replace the `src` with a single file or directory, and the relevant file(s) will be formatted automatically according to our current prettier standards. You can replace the `--write` with `--check` to simply check formatting without making changes. There's also

**- JSBeautify (HTML)**. We use [JSBeautify](https://beautifier.io/) to format HTML files. You can use `npm run format:html` to format all HTML files in the repository. You can also use `npm run format:html-file [FILE]` to format a single file.

**_Run All Formatting_**. Use `npm run format:all` and all TS, CSS, and HTML files inside the `src` directory will be formatted automatically for you. This is command you should automate or run prior to creating your Pull Requests.


# Other Commands

In addition, the following npm script commands are available when working on this application:

**- Code Formatting (HTML)**. Use `npm run format -- '[FILE]'` on an HTML template file named *FILE*, and the file will be formatted automatically according to our current linting standards. You can also run `npm run format: all` to reformat all html files of the application at once.

**- Code Linting (TS)**. Use `npm run lint --files '[FILE]'` on a TS script file named *FILE* to see lint errors for that file (but make no changes to it). Append `--fix` to this command to fix *some* of the issues automatically for you.


# Directory Structure

Coming soon.

# Featured Components

## Data-Table
This custom component handles most of the many data tables used in the Umbrella so far.

Below is a list of the `@input` parameters the component can handle for customization and configuration of a data table:

  * **tableId**: _string_ - A unique ID for the data-table
  * **columns**: _any_[] - An array of columns, each of type [shared/types/data-table-column.type](DataTableColumn), specifying the columns of data to display
  * **service**: _any_ - The service that the data-table uses to fetch and update data from the API
  * **dataSource**: _any_ - The datasource file that enables the Material table to function
  * **dataSet**: _any_[] - Customize via filters what data is fetched from the API and appears in the table.
  * **dropdownOptions**: _any_[] - An object configuring the drop-down options available for each row in the table when clicking the meatballs icon
  * **search**: _boolean_ - Whether or not the table should offer a search bar
  * **searchFields**: _string_ - A comma-delimited list of the searchable fields
  * **filter**: _boolean_ - Whether or not the table should offer filtering UI
  * **filters**: _any_ - An object configuring which filters are available to the user, and whether they are preselected or not.
  * **lookupTables**: _any_ - Tables that can be used to display data rather than simply IDs that are part of a row of the data
  * **actionButton**: _boolean_ - Whether or not the table should offer action buttons, such as Delete or Edit, atop the table
  * **buttonActions**: _any_ - An object configuring the actions available when `actionButtons` is set to `true`
  * **checkboxColumn**: _boolean_ - Whether or not the table should offer an ID column of checkboxes, enabling multi-select of the rows of the table
  * **statusColumn**: _boolean_ - Whether or not to show a status column
  * **width**: _string_ - Whether the table should be narrow or not. Use `'narrow'` if so.

  As for display, all data is displayed as standard `Ant` table data unless a `format` field is present when the column is defined. If a `format` field is defined, these are the options:

  * **date** - The data will be displayed in the table as a date.
  * **twoFields** - Two fields will be concatenated as strings. Useful for a full name field, for instance.
  * **boolean** - A boolean data field will be displayed with one of two defined strings, depending on whether the data is `true` or `false`.
  * **label** - Data can be displayed as colored labels. Pass into the `format` object `format.type = 'label'` and a `format.labels` *object* with *key(s)* that are the value(s) you want to display labels for, and the *value(s)* are mini-objects containing `labelText` and `labelColor` for each value. The colors currently available are `green`, `red` and `gray`.
  * **lookupTable** - A data table can be used to display text depending on the data for each row. This data table can be loaded asynchronously in the `makeLookupTables()` component function.
  * **image** - An image can be displayed, as defined in the `format.src` field.

# Extending the Application

## Contributing

To add features or fix bugs in this app, always:

* Use the architecture and directory structure, as discussed above, and add new components in the proper place so that engineers who come after you know where to find things.
* Use the automatically-generated Code Scaffolding, as described above, to generate new components whenever needed
* Use Prettier. Utilize formatting and linting to add code that is compliant with standards.
* Use the `data-table` component when you need to add a new table of data.
* Write unit tests for your components.
* Write your Pull Request as specified below.
* Update this README if and when necessary.
* Follow the Git branching strategy above to create your branches.
* That's it! 🎉


### Pull Requests

An optimal PR for our git history explains all of the changes that it makes to the repository with a title and description that is easy to read and to
understand, and (if necessary) to revert.

#### PR Title

The title of the PR should be a very brief imperative-voice description of what the PR does,
prefixed by the JIRA story abbreviation it addresses. (If there is no JIRA ticket,
use the component that the pull request affects or "[MAINT]" if the PR addresses repository
maintenance):

{{ [JIRA-XXX] }} - {{ Brief summary of what PR does }}

eg:

    [FS-326] - Fix settings menu

If a pull request is **not** to be considered for merging (yet), please prefix the PR title with [WIP] or [DO NOT MERGE] to indicate the code isn't intended to be merged.

#### PR Body

While this isn't required, the body of the pull request ideally helps the next engineer review the code by following the _Problem/Solution/Result_ pattern. Each section
should contain a description or list of items. The _Solution_ section should describe what each change does, together with any justifications, reasoning, or concerns. Include
a link to the JIRA ticket when possible in the _Problem_ section.

eg:
```
    PROBLEM
    - Description (eg: Can’t view the menu)
    - [JIRA ticket link]

    SOLUTION
    - What you did, what files were edited, why
    - (eg: Fixed menu visibility)

    RESULT
    - User experience change
    - (eg: Menu now appears on click)
    - Any changes engineers will experience
    - If changes to UI are substantial or important, include screenshot of the app showing changes to the UI.
```

A cool feature of BitBucket is that the commit message for
the first commit of a PR will automatically become the PR Title and Message, so if you write a great commit message, following the rubric above, it will automatically become your PR message.

We use the `squash` method to merge PRs by default. This keeps a cleaner commit history in the `develop` and other mainline branches. If you want to use a different merge strategy, select it when creating your PR. **Note: When merging into `develop` or another mainline branch, please delete the top line of the default merge commit comment so that the PR title is what is listed first. **

#### PR Review Checklist

When you review another developer's PR, please ensure these basic things are present:

  - [ ] PR is into the proper branch, and is in sync with that branch
  - [ ] The pipeline build on the PR is passing.
  - [ ] PR Title follows the title standard.
  - [ ] There is at least a basic description of the PR's purpose in the description (Not every PR description has to have every component listed above).
  - [ ] There is unit testing present, or an explanation as to why it isn't present.

## Styles

Global styles are in the wrapper module's [component style sheet](src/app/user-management/user-management.component.scss). All component-specific style rules should go into that component's `.scss` styles file.

### Utility Classes
A large and useful set of utility classes are generated by a [SASS mix-in](src/app/user-management/user-management.component.scss#109) inside our root styles file. Please use these classes when possible, rather than create your own new style rules.

For instance, rather than creating a new class or adding a rule to an existing class for a left-margin of 10px, simply add the `margin-left-10px` class to your element:

``` <div class="margin-left-10px">Sample Content</div>```

Use these utility classes where possible so as to write as few new rules as possible.

### Nesting
Also, use [SCSS nesting](https://www.w3schools.com/sass/sass_nesting.asp) inheritance to organize and simplify your code.

## Code Comments
### Explanations
Generally code should be clear and not complex so that inline documentation and explanation is not necessary. However, whenever the code is complex or uses a solution or algorithm that will not be readily apparent to another engineer who might review or extend the code, always add comments explaining the solution or algorithm used. `/* */` comment notation should be used for these explanatory comments.

### Unused code
Code that is currently unneeded should generally be removed from the repository. There is no need to keep unused code as the git history will keep it. However, during initial buildout of features or when the API integration is unstable or not yet used, there will be times to **temporarily** commment out code. `//` comment notation should be used. This way code comments is immediately differentiated from unneeded code.

### TODOs
Tasks that should be completed soon and that are not captured in JIRA tickets can be noted with JSDoc's `/* @TODO */` notation. This way a quick search on the @TODO will bring up all the tasks flagged in the code. Also if we utilize JSDoc in the future, the @TODO notation will be helpful.

# Monitoring

Information on site monitoring coming soon:
- Health Check
- NodePing
- New Relic

# Further help

To get more help on the Angular CLI use `ng help` or review the [Angular CLI README](https://github.com/angular/angular-cli/blob/master/README.md).

